// Gary Yngve 10/10/2005
// Looks at the adjacent squares to see if a decision can be made
// based on counting the number of mines/empties around an askable
// square versus the value from ask

public class CountMineStrategy implements MineStrategy {

    public static final CountMineStrategy INSTANCE = new CountMineStrategy();

    private CountMineStrategy() {} // singleton

    // if (i,j) is askable, counts the number of mines and empties
    // to see if all the remaining squares are either empties or mines
    private int testNeighbor(MineBoard mb, int i, int j) {
        if (mb.askable(i,j)) {
            int k = mb.ask(i,j);
            int marked = 0, empty = 0;
            for(int m = -1; m <= 1; m++)
                for(int n = -1;n <= 1; n++) {
                    if (m == 0 && n == 0)
                        continue; // don't count the empty middle square
                    if (mb.isMarked(i+m,j+n))
                        marked++;
                    if (mb.isOOB(i+m,j+n) || !mb.isUnknown(i+m,j+n))
                        empty++;
                }
            if (marked == k) // accounted for all mines
                return MineStrategy.EXPLORE;
            if (8-empty == k) // only mines left
                return MineStrategy.MARK;
        }
        return MineStrategy.UNKNOWN;
    }

    public int decide(MineBoard mb, int i, int j) {
        for(int m = -1; m <= 1; m++)
            for(int n = -1; n <= 1; n++) { // try all neighbors
                // to see if they are askable.
                // we know (0,0) is not askable
                int ans = testNeighbor(mb,i+m,j+n);
                if (ans != MineStrategy.UNKNOWN)
                    return ans; // return when we find something decisive
            }
        return MineStrategy.UNKNOWN;
    }

}
