// Gary Yngve 10/10/2005
// We want to deduce information about (i,j).  So we create a bunch of
// possible layouts around (i,j), discard the ones that don't satisfy the
// constraints, and analyze the statistics of the remaining ones.  If we
// can conclude (with probability 1-epsilon) that (i,j) is safe or a mine,
// then return the corresponding strategy.

public class DeriveRuleMineStrategy implements MineStrategy {

    private int numGuesses; // number of trials that pass constraints
    private int winSize; // best with 2

    public DeriveRuleMineStrategy(int numGuesses) {
        this(2,numGuesses);
    }

    public DeriveRuleMineStrategy(int winSize, int numGuesses) {
        this.winSize = winSize;
        this.numGuesses = numGuesses;
    }

    public int decide(MineBoard mb, int x, int y) {
        GuessWindow g = new GuessWindow(winSize);

        // for the sake of eliminating rotational symmetries
        // and reducing unneeded information, use a 5x5 window
        // but have the deciding square be off-center.  Do for all
        // four quadrants.
        for(int i = -1; i <= 1; i+=2)
            for(int j = -1; j <= 1; j+=2) {
                int sum = 0;
                for(int k = 0; k < numGuesses; k++) {
                    g.populateValidLayout(mb,x+i,y+j);
                    if (g.get(-i,-j)) // offset back to (0,0)
                        sum++;
                }
                if (sum == 0)
                    return MineStrategy.EXPLORE;
                else if (sum == numGuesses)
                    return MineStrategy.MARK;
            }
        return MineStrategy.UNKNOWN; // only decide if 100% sure
    }
}
