// Gary Yngve 10/10/2005
// Solves an endgame situation.  All unknown squares are stored in a list,
// since they are sparsely located on the board.  Generates random samples
// using knowledge of the remaining number of mines.  Discards the ones that
// don't satisfy the constraints (e.g. 3 mines around my neighborhood), and
// analyzes the statistics.
// Provides the locations that are guaranteed mines or safe.

import java.awt.Point;
import java.util.*;

public class EndGame {

// A possible square to explore/mark.  Has a location
// and a running count of number of mines sampled there.
    private static class EndGameChoice {
        public EndGameChoice(Point loc) {
            this.loc = loc;
        }
        public Point loc;
        public int count;
    }

// technically, unknowns and asks could be parameters repeatedly passed
// around, but for convenience, and for directing you toward the solution,
// they are defined here
    private List unknowns; // List of all unknown squares on the board.
    private List asks; // List of all relevant askable squares on the board.
    private boolean[][] mines; // same size as MineBoard, true means mine
    private MineBoard mb;
    private int numSamples; // number of trials

// assigns the parameters to state, fills in unknowns and asks,
// initializes mines
    public EndGame(MineBoard mb, int numSamples) {
        this.mb = mb;
        this.numSamples = numSamples;
        unknowns = new ArrayList();
        asks = new ArrayList();
        mines = new boolean[mb.width()][mb.height()];
                        // 2-d array, same dimensions as mb
        for(int i = 0; i < mb.width(); i++)
            for(int j = 0; j < mb.height(); j++) { // for each square (cell)
                throw new RuntimeException("not implemented");
            }
    }

// upon completion, toExplore and toMark are respectively
// filled with Points to be explored/marked.
// Must use iterators for full credit.
// ANALYZE IS INVALID IF MINEBOARD HAS CHANGED SINCE CONSTRUCTION:
// MAKE A NEW INSTANCE OF ENDGAME IF THE BOARD HAS CHANGED
// analyze adds to whatever was already in the lists (doesn't clear them)
    public void analyze(List toExplore, List toMark) {
        throw new RuntimeException("not implemented yet");
    }

// put additional methods here

// an askable cell is only relevant if it borders an unknown cell
    private boolean relevantAsk(int i, int j) {
        boolean b = false;
        for(int m = -1; m <= 1; m++)
            for(int n = -1; n <= 1; n++)
                if (mb.isUnknown(i+m,j+n) && !mb.isMarked(i+m,j+n))
                    b = true;
        return b;
    }

// returns the number of neighbors (8-connected) on the boolean mines array
// that are true (have mines)
    private int numNeighborsTrue(int i, int j) {
        if (!mb.inBounds(i,j))
            throw new IllegalArgumentException("i,j = " + i + "," + j
                                               + " is OOB");
        else {
            int sum = 0;
            for(int m = -1; m <= 1; m++)
                for(int n = -1; n <= 1; n++) {
                    if (m == 0 && n == 0)
                        continue; // not needed if (i,j) askable
                    if (mb.inBounds(i+m,j+n) && mines[i+m][j+n])
                        sum++;
                }
            return sum;
        }
    }
}
