// Gary Yngve 10/10/2005
// Implements a 3x3 inner window that can ask, and a 5x5 window that holds mines
// It generates random mine layouts that are constrained to the numbers in the
// 3x3 area. (when size == 2)

import java.util.*;

public class GuessWindow {

    private static Random r = new Random();
    private boolean [][]vals; // the window, centered at (0,0)
    private int size; // radius of window

    public GuessWindow(int size) {
        vals = new boolean[2*size+1][2*size+1];
        this.size = size;
    }

// produces a layout that does not violate any information
// available from the board around (x,y)
    public void populateValidLayout(MineBoard mb, int x, int y) {
        do { // rejection method
            populatePossibleLayout(mb,x,y);
        } while (!isValidLayout(mb,x,y));
    }

    // generates  a layout with trues at the Marked cells, falses
    // at the Explored Safely cells, and random 50-50 otherwise.
    private void populatePossibleLayout(MineBoard mb, int x, int y) {
        for(int i = -size; i <= size; i++)
            for(int j = -size; j <= size; j++) {
                if (mb.isMarked(x+i,y+j))
                    set(i,j,true);
                else if (mb.isUnknown(x+i,y+j))
                    set(i,j,r.nextBoolean());
                else set(i,j,false);
            }
    }

    // returns true if the layout satisfies all the constraints
    private boolean isValidLayout(MineBoard mb, int x, int y) {
        for(int i = -(size-1); i <= (size-1); i++)
            for(int j = -(size-1); j <= (size-1); j++) {
                if (mb.askable(x+i,y+j)) { // for each constraint in inner win
                    if (mb.ask(x+i,y+j) != numNeighborsTrue(i,j))
                        return false; // failed the constraint
                }
            }
        return true;
    }

    // centered at (0,0)
    public  boolean get(int i, int j) {
        return vals[i+size][j+size];
    }

    // centered at (0,0)
    private void set(int i, int j, boolean val) {
        vals[i+size][j+size]=val;
    }

// count the number of mines adjacent (8-neighbors) to (i,j)
// undefined behavior if (i,j) is a mine.
// (throws an exception if not not in the inner window)
    private int numNeighborsTrue(int i, int j) {
        if (i < -(size-1) || i > (size-1)
            || j < -(size-1) || j > (size-1))
            throw new IllegalArgumentException("i,j = " + i + "," + j
                                               + " is OOB");
        else {
            int sum = 0;
            for(int m = -1; m <= 1; m++)
                for(int n = -1; n <= 1; n++)
                    if (get(i+m,j+n)) sum++;
            return sum;
        }
    }
}
